;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look for lisp packages where none of the "Keywords:" entries are in
;; `finder-known-keywords' and for that reason won't appear under
;; `M-x finder-by-keyword'.


(require 'cl)
(require 'finder)
(require 'lisp-mnt)

(defun my-keyword-known (str)
  (member* str finder-known-keywords
           :test 'equal
           :key (lambda (elem)
                  (symbol-name (car elem)))))

;; (my-keyword-known "c")

(defun my-locate-list (pattern)
  "Return a list of filenames from `locate' matching PATTERN."
  (require 'locate)
  (with-temp-buffer
    (let ((default-directory "/")
          (process-connection-type nil) ;; pipe
          (coding-system-for-read (or file-name-coding-system
                                      default-file-name-coding-system)))
      (call-process locate-command
                    nil ;; input /dev/null
                    t   ;; output to buffer
                    nil ;; no redisplay
                    "-0"
                    "--"
                    pattern)
      (split-string (buffer-string) "\0" t))))
;; (my-locate-list ".sqdb")                    
;;   (locate pattern)
;;   (let (ret)
;;     (while (re-search-forward "^    \\(/.*\\)" nil t)
;;       (push (match-string 1) ret))
;;     (kill-buffer nil)
;;     ret))

(defun my-lm-keywords-list (filename)
  (with-temp-buffer
    (insert-file-contents filename)
    (lm-keywords-list)))

;; (my-lm-keywords-list "/down/emacs/lisp/finder.el")

(defun my-keywords-check-one (filename)
  (unless (lm-keywords-finder-p filename)
    (lwarn 'emacs-lm-keywords 'warning "%s none known: %S"
           filename
           (my-lm-keywords-list filename))))

;;   (let ((keyword-list (my-lm-keywords-list filename)))
;;     (unless (or (null keyword-list)
;;                 (some 'my-keyword-known keyword-list))
;; ))))

;; (my-keywords-check-one "/down/emacs/lisp/finder.el")

(let ((count 0)
      (emacs-lisp-mode-hook nil))
  (dolist (filename (append
                     ;; (my-locate-list "*.el")
                     ;; (my-locate-list "*.el.gz")
                     (file-expand-wildcards "/down/emacs/lisp/*.el")
                     (file-expand-wildcards "/down/emacs/lisp/*/*.el")
                     (file-expand-wildcards "/down/emacs/lisp/*/*/*.el")
                     ))
    (my-keywords-check-one filename))
  (lwarn 'emacs-lm-keywords 'warning "done %d" count))
