;; Copyright 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  ;; this skip depends on ffap-rfc-directories.el not loaded yet
  (require 'ffap)
  (if (boundp 'ffap-rfc-directories)
      (message "skip, `ffap-rfc-directories' builtin in this emacs")

    (let ((count 0)
          (tempbuf "*t-ffap-rfc-directories--temp*"))
      (switch-to-buffer tempbuf)

      ;;-----------------------------------------------------------------------
      ;; load

      (require 'ffap-rfc-directories)

      ;; no leftovers in emacs21 byte compiling defadvice
      (when (and (boundp 'byte-compile-unresolved-functions)
                 byte-compile-unresolved-functions)
        (error "byte-compile-unresolved-functions after load: %S"
               byte-compile-unresolved-functions))
      (setq count (1+ count))


      ;;-----------------------------------------------------------------------

      (let ((want (expand-file-name "rfc0000.txt")))
        (setq ffap-rfc-directories (list (expand-file-name ".")))
        (setq ffap-rfc-path "/went/to/path/variable/%s.txt")
        (insert "RFC0000")
        (goto-char (point-min))
        (let* ((guess (ffap-guesser))
               (got   (expand-file-name guess)))
          (unless (equal want got)
            (error "Visiting RFC0000 got %S want %S" got want)))
        (setq count (1+ count)))

      ;;-----------------------------------------------------------------------
      ;; unload-feature

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)

      (let* ((feature 'ffap-rfc-directories)
             (adv     (my-advice-by-name feature)))

        (unload-feature feature t)

        (cond ((not adv)
               (message "skip, no defadvice in this emacs"))
              ((featurep 'xemacs)
               (message "skip, xemacs doesn't unload-hook"))
              (t
               (let ((lst (my-advice-by-name feature)))
                 (when lst
                   (error "unload-feature leftover advice: %S" lst))
                 (setq count (1+ count))))))

      (when (boundp 'ffap-rfc-directories)
        (error "variable `ffap-rfc-directories' not unloaded"))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      ;; unload-feature -- with advice unloaded first

      (let ((feature 'ffap-rfc-directories))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      (message "  %d tests" count)
      count)))
