;; Copyright 2010, 2012 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(formfeed-hline-mode))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'formfeed-hline)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; enable/disable

    (message "t-formfeed-hline.el: enable")
    (formfeed-hline-mode 1)
    (setq count (1+ count))

    (message "t-formfeed-hline.el: disable")
    (formfeed-hline-mode 0)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; xemacs defadvices

    (unless (equal emacs-version "24.1.1") ;; avoid `delete-window' segv
      (delete-window (split-window)))
    (setq count (1+ count))

    (set-window-configuration (current-window-configuration))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; formfeed-hline-char-table-equal

    (if (featurep 'xemacs)
        (message "skip formfeed-hline-char-table-equal in xemacs")

      (let ((table1 (make-display-table))
            (table2 (make-display-table)))
        (unless (formfeed-hline-char-table-equal table1 table2)
          (error "formfeed-hline-char-table-equal: two make-display-table's"))
        (setq count (1+ count)))

      (let ((table1 (make-display-table))
            (table2 (make-display-table)))
        (aset table1 ?X [?#])

        (when (formfeed-hline-char-table-equal table1 table2)
          (error "formfeed-hline-char-table-equal: unequal with ?X 1-2"))
        (setq count (1+ count))

        (when (formfeed-hline-char-table-equal table2 table1)
          (error "formfeed-hline-char-table-equal: unequal with ?X 2-1"))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no FEATURE-unload-function")

      (let ((feature 'formfeed-hline))
        (when (featurep 'xemacs)
          (unless (my-advice-by-name feature)
            (error "oops, didn't find any advice")))

        (formfeed-hline-mode 1)

        (message "t-formfeed-hline.el: attempt unload-feature")
        (unload-feature feature t)
        (setq count (1+ count))

        (when (memq 'formfeed-hline-window-size-change
                    window-size-change-functions)
          (error "unload-feature leftover in window-size-change-functions: %S"
                 window-size-change-functions))

        ;; try defanged xemacs defadvices when unloaded
        (set-window-configuration (current-window-configuration))
        (setq count (1+ count))

        (unless (equal emacs-version "24.1.1") ;; avoid `delete-window' segv
          (delete-window (split-window)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- `window-size-change-functions' still works

    ;; in emacs21 up window-size-change-functions is purged by
    ;; unload-feature, but not in xemacs21, so manually disable
    (require 'formfeed-hline)
    (formfeed-hline-mode 1)
    (formfeed-hline-mode 0)
    (message "t-formfeed-hline.el: window-size-change-functions before unload %S"
             window-size-change-functions)
    (unload-feature 'formfeed-hline t)
    (message "t-formfeed-hline.el: window-size-change-functions after unload  %S"
             window-size-change-functions)
    (run-hook-with-args 'window-size-change-functions (selected-frame))
    (setq count (1+ count))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'formfeed-hline))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

