;; Copyright 2014, 2019 Kevin Ryde

;; This file is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; This file is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this file; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;; stock-quote.el from
;; http://www.newartisans.com/johnw/emacs.html
;; http://web.archive.org/web/20060210095105/www.newartisans.com/johnw/Emacs/obsolete/stock-quote.el


;; Alas, Yahoo Finance no longer has this CSV interface.

;; As of Jan 2019 it has two types of JSON, one with cookies and one
;; without, but both needing parsing and truncating of decimals.
;; See my perl Chart program for code, or maybe hopefully perhaps
;; perhaps Finance::Quote too.

(defun my-stock-quote-yahoo (ticker)
  "Download a stock price using Yahoo Finance (CSV format).
This is designed for use in `stock-quote-data-functions' of
stock-quote.el."

  (require 'url)
  (with-temp-buffer
    (let (url-show-status)
      (url-insert-file-contents
       ;; s=FOO is the stock, f=sl1 asks for stock and last price
       (concat "http://quote.yahoo.com/d?f=sl1&s=" ticker))
      (set-buffer-modified-p nil))

    ;; yahoo gives "N/A" for an unknown ticker, which comes out as 0
    ;; from string-to-number, maybe throwing an error would be better
    (goto-char (point-min))
    (re-search-forward (concat (regexp-quote ticker) "\",\\(.+\\)"))
    (string-to-number (match-string 1))))

(custom-add-option 'stock-quote-data-functions 'my-stock-quote-yahoo)

